#!/usr/local/filewave/python/bin/python

import argparse
import base64
import csv
import os
import re
import requests
import sys


"""
Author: Kevin Hunter Kesling
Created: 2019-07-24
"""

bname = os.path.basename( sys.argv[ 0 ])

usage = f"""{bname} --token <auth_token> --host <fw_host> --mapping /some/path.csv

This script is meant to mimic the "Change Enrollment Username" context menu
item in FW Admin.  Whereas that interface requires a human to change a device
at a time, this tool reads a CSV to do the work in bulk.  Example CSV file:

    $ cat example.csv
    Device ID,Enrollment Username
    7sm3qvzxwj1uxoy6,alice
    g4r7fnmdsa7c3d1v,bob
    p4tb56i1qmn4na81,charlie

The first row, the "Title Row" is mandatory, serving as basic human check on
"did you select the correct file?"  It must match the example.  The device ids
and usernames should match the environment in question.  In particular, the
device ids would match the data in

    FW Admin -> Device Details (tab) -> Device ID (row)

There is no limit on the number of rows; suggest building this in a spreadsheet
and exporting as CSV.

Executing this script:

    $ {bname}  --host <host> --token <token> --mapping /some/path/to/your.csv
"""


# For the further-inclined, note that this is a wrapper around the
# `/api/device_auth_username` endpoint; if you want to use Curl instead, the CLI
# might be:
#
# curl -vsk https://<host>:20443/api/device_auth_username -H "Authorization: e2..." -H "Content-Type: application/json" --data '{{"device_id": "...", "new_auth_username": "..."}}'

parser = argparse.ArgumentParser( usage=usage )
parser.prog = os.path.basename( bname )

parser.add_argument('--mapping',
  type=str,
  required=True,
  help='CSV file containing (per row) a DEP user to create.  (Note: first row should be title, so will be skipped.)'
)
parser.add_argument('--host',
  type=str,
  required=True,
  help='Specify the hostname against which to run the API calls.'
)
parser.add_argument('--token',
  type=str,
  required=True,
  help='The authorization token to allow API usage.  This is the Shared Key (including braces) from Preferences -> Mobile (tab).'
)
parser.add_argument('-p', '--port',
  type=int,
  default=20443,
  help='Specify the port to use.  [Default: 20443].'
)
# parser.add_argument('-d', '--device_id',
#   type=str,
#   help='Specify a device id for a one-off use of this script.'
# )
# parser.add_argument('-u', '--username',
#   type=str,
#   help='Specify a username for a one-off use of this script.'
# )

uri_path = '/api/device_auth_username'
args = parser.parse_args( sys.argv[1:] )

if not os.path.exists( args.mapping ):
    raise SystemExit(f'[{args.mapping}] Unable to find file.  Path typo?')

url = f'https://{args.host}:{args.port}{uri_path}'
headers = {
    'Authorization': f'{base64.encodebytes(args.token.encode()).decode().strip()}'
}
expected_header = ['Device ID', 'Enrollment Username']
bookend_whitespace_re = re.compile(r'^\s+|\s+$')

noted_usernames = []
unexpected_success = []
failures = []

# workaround use of self-signed certs:
requests.packages.urllib3.disable_warnings(requests.packages.urllib3.exceptions.InsecureRequestWarning)
with open(args.mapping, 'r') as csv_file:
    csv_reader = csv.reader( csv_file )
    header = next( csv_reader )
    if header != expected_header:
        parser.print_help()
        raise SystemExit(f'[{fpath}] Invalid CSV.  Did not find expected header row.')

    for row_num, row in enumerate(csv_reader, start=2):
        if len( row ) != len( expected_header ):
            continue

        device_id, username = row
        device_id = device_id.strip()

        if bookend_whitespace_re.match( username ):
            noted_usernames.append((row_num, device_id))

        # gleaned from reading the actual API endpoint
        data = {'device_id': device_id, 'new_auth_username': username}
        res = requests.post(url, json=data, verify=False, headers=headers)
        if 200 <= res.status_code and res.status_code < 300:
            if 204 != res.status_code:
                unexpected_success.append( device_id )
            else:
                sys.stderr.write(f'Successfully updated device: {device_id}\n')
        else:
            failures.append( device_id )

if unexpected_success:
    sys.stderr.write('Server reported success for these devices, but gave an "unexpected" success\n'
                     'response.  Please confirm these records:\n\n')
    for device_id in unexpected_success:
        sys.stderr.write(f'  {device_id}\n')

if failures:
    sys.stderr.write('Failed to update these devices:\n\n')
    for device_id in failures:
        sys.stderr.write(f'  {device_id}\n')

if noted_usernames:
    for row_num, un in noted_usernames:
        sys.stderr.write(f'NOTICE: (row {row_num}) Specified username had extra whitespace.  Typo?  {un}')
